import tempfile
import json

class Tester(object):
    def __init__ (self, runner):
        self.runner = runner

    def test_config_structure (self, node, config=None):
        result = { "valid": False }

        config = config if config else node.json["config"]
        with self.runner.open() as pa:
            @pa.on("FIND_PLUGINS_RESPONSE")
            def onPluginsFound (notification):
                pa.sendNotification("VERIFY_PLUGIN_CONFIG", {
                    "pluginID": config["pluginID"],
                    "config": json.dumps(config)
                })

            @pa.on("VERIFY_PLUGIN_CONFIG_RESPONSE")
            def onVerifyPluginConfig (notification):
                print(notification)
                body = json.loads(notification["body"])
                result["valid"] = body["valid"]
                pa.terminate()
        
        return result["valid"]

    def test_authentication (self, node, config=None):
        result = { "result": None }
        root = tempfile.mkdtemp()

        config = config if config else node.json["config"]
        with self.runner.open() as pa:
            @pa.on("FIND_PLUGINS_RESPONSE")
            def onPluginsFound (notification):
                pa.sendNotification("GRAPH_BUILDER_MAKE", {
                    "root": root
                })

            @pa.on("GRAPH_BUILDER_MAKE_RESPONSE")
            def onGraphBuilderMake (notification):
                pa.sendNotification("GRAPH_BUILDER_CREATE_NODE", {
                    "root": root,
                    "pluginID": config["pluginID"],
                    "config": config
                })

            @pa.on("GRAPH_BUILDER_CREATE_NODE_RESPONSE")
            def onGraphBuilderCreateNode (notification):
                pa.sendNotification("GRAPH_BUILDER_INVOKE_SERVICE", {
                    "root": root,
                    "nodeID": 1,
                    "service": "verifyCredentials",
                    "arguments": {}
                })

            @pa.on("GRAPH_BUILDER_INVOKE_SERVICE_RESPONSE")
            def onGraphBuilderInvokeService (notification):
                body = json.loads(notification["body"])
                result["result"] = body.get("result", None)
                pa.terminate()
        
        return result["result"]