import subprocess
import os
import json

from . import terminal_exec

VIEWER_COMMAND = "less"

LOGDUMP_FILE_PATTERN = "{pluginId}-{nodeId}.log"
PROCESSOR_LOG_FILE = "processor.log"

class LogViewer(object):
    def __init__ (self, terminal_executor=None, viewer_command=VIEWER_COMMAND):
        self.terminal_executor = terminal_executor if terminal_executor else terminal_exec.TerminalExecutor()
        self.viewer_command = viewer_command

    def __call__ (self, log_file):
        self.terminal_executor.execute([self.viewer_command, log_file])

class LogFinder(object):
    def __init__ (self, graph_path=None, node=None):
        self.root = None
        if graph_path:
            self.setGraphPath(graph_path)

        if node:
            self.setNode(node)

    def setNode (self, node):
        if node is None:
            return
        
        self.node = node
        self.config = json.loads(node["config"])

    def setGraphPath (self, graph_path):
        self.graph_path = graph_path
        self.root = os.path.dirname(self.graph_path)

    def __call__ (self, node=None):
        self.setNode(node)
        if self.root is None:
            return

        log_file = self.find_log_file_from_logdump()
        if log_file:
            return log_file

        log_file = self.find_log_file_from_capture_session()
        if log_file:
            return log_file        

    def find_processor_directory (self, node=None):
        self.setNode(node)
        if self.root is None:
            return
        
        processor_directory = os.path.join(
            self.root, self.config["pluginID"], str(self.node["nodeID"])
        )

        if os.path.isdir(processor_directory):
            return processor_directory

    def find_log_file_from_logdump (self, node=None):
        self.setNode(node)
        if self.root is None:
            return
        
        log_file = os.path.join(self.root, LOGDUMP_FILE_PATTERN.format(
            pluginId=self.config["pluginID"], nodeId=self.node["nodeID"]
        ))

        if os.path.isfile(log_file):
            return log_file

    def find_log_file_from_capture_session (self, node=None):
        self.setNode(node)
        if self.root is None:
            return

        processor_directory = self.find_processor_directory()
        if processor_directory is None:
            return
        
        log_file = os.path.join(processor_directory, PROCESSOR_LOG_FILE)
        if os.path.isfile(log_file):
            return log_file