import os
import tempfile
import json

from . import graph_runner

class OutputChecker(object):
    def __init__ (self, runner):
        self.runner = runner

    def get_output_of_type (self, graph, node, output_type, config=None):
        result = { "result": None }
        root = tempfile.mkdtemp()

        config = config if config else node.json["config"]

        graph_runner.dump_graph(graph, root)

        with self.runner.open() as pa:
            @pa.on("FIND_PLUGINS_RESPONSE")
            def onPluginsFound (notification):
                pa.sendNotification("GRAPH_BUILDER_CONFIGURE", {
                    "root": root,
                    "nodeID": node.json["nodeID"],
                    "config": config
                })

            @pa.on("GRAPH_BUILDER_CONFIGURE_RESPONSE")
            def onGraphBuilderConfigureNode (notification):
                pa.sendNotification("GRAPH_BUILDER_GET_OUTPUT", {
                    "root": root,
                    "nodeID": node.json["nodeID"],
                    "type": output_type.upper().replace(" ", "_")
                })

            @pa.on("GRAPH_BUILDER_GET_OUTPUT_RESPONSE")
            def onGraphBuilderGetOutput (notification):
                body = json.loads(notification["body"])
                result["result"] = body.get("result", None)
                pa.terminate()
        
        return result["result"]

    def get_output_json (self, node):
        if node.processor_directory is None:
            return

        output_json = os.path.join(node.processor_directory, "output.json")
        if not os.path.isfile(output_json):
            return

        return json.load(open(output_json))